package tv.smartclip.smartclientandroid.app.reference

import android.annotation.SuppressLint
import android.app.AlertDialog
import android.os.Bundle
import android.view.ViewGroup
import dev.zieger.utils.coroutines.builder.launchEx
import kotlinx.android.synthetic.main.activity_in_stream.*
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.Job
import tv.smartclip.smartclientandroid.lib.SxSequencer
import tv.smartclip.smartclientandroid.lib.dto.SxConfiguration
import tv.smartclip.smartclientandroid.lib.dto.SxLayoutConfiguration
import tv.smartclip.smartclientandroid.lib.dto.sequencer.SxSequencerAdSlot
import tv.smartclip.smartclientandroid.lib.dto.sequencer.SxSequencerRelativePosition
import tv.smartclip.smartclientandroid.lib.video_player.InstreamExoWrapper
import kotlin.coroutines.CoroutineContext

/**
 * Displays a video in a SimpleExoPlayer that plays after an advertisement's video.
 * Uses the [InstreamExoWrapper].
 */
@SuppressLint("LogNotTimber")
class InstreamExoWrapperActivity : BaseActivity() {

    private val mainScope: CoroutineScope = object : CoroutineScope {
        override val coroutineContext: CoroutineContext = Dispatchers.Main + Job()
    }

    private val contentVideo: String
        get() = getString(R.string.demo_video_short)
    private lateinit var playerWrapper: InstreamExoWrapper
    private lateinit var sequencer: SxSequencer
    private val adSlots: List<SxSequencerAdSlot>
        get() = listOf(
                createAdSlot(0.0),
                createAdSlot(0.33),
                createAdSlot(0.66),
                createAdSlot(1.0)
        )

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_in_stream)

        playerWrapper = InstreamExoWrapper(video, contentVideo)
        sequencer = SxSequencer(playerWrapper, adSlots, getConfiguration())
        playerWrapper.playWhenReady = true

        title = getString(R.string.title_instream_exo)
    }

    private fun createAdSlot(position: Double) =
            SxSequencerAdSlot(getString(R.string.ad_tag_short),
                    SxSequencerRelativePosition(position),
                    getString(R.string.default_opener_url),
                    false,
                    getString(R.string.default_closer_url),
                    false,
                    getString(R.string.default_bumper_url),
                    false)


    private fun getConfiguration() = SxConfiguration.INSTREAM.apply {
        layoutConfiguration = SxLayoutConfiguration(
                width = ViewGroup.LayoutParams.MATCH_PARENT,
                height = 0,
                orientation = SxLayoutConfiguration.SxOrientation.HORIZONTAL
        )
        skipAdDuration = 1
        clickThroughListener = { callback ->
            mainScope.launchEx {
                AlertDialog.Builder(this@InstreamExoWrapperActivity)
                        .setTitle("Custom Click Through Dialog")
                        .setMessage("Do you want to proceed?")
                        .setPositiveButton("Yes") { _, _ -> callback(true) }
                        .setNegativeButton("No") { _, _ -> callback(false) }
                        .setOnCancelListener { callback(false) }
                        .show()
            }
            true
        }
    }

    override fun onResume() {
        super.onResume()
        playerWrapper.playWhenReady = true
    }

    override fun onPause() {
        playerWrapper.playWhenReady = false
        super.onPause()
    }

    override fun onDestroy() {
        sequencer.release()
        super.onDestroy()
    }
}